package PiTube::Controller::Websockets;

use feature 'signatures';
no warnings qw{ experimental::signatures };

use Mojo::Base 'Mojolicious::Controller';
use Mojo::Pg::PubSub;

use constant SOCKET_INACTIVITY_TIMEOUT => 300;

sub main {
    my $c = shift;

    $c->inactivity_timeout(SOCKET_INACTIVITY_TIMEOUT);

    if ($c->req->headers->header('Sec-WebSocket-Extensions') =~ /permessage-deflate/) {
        $c->tx->compressed(1);
        $c->res->headers->add('Sec-WebSocket-Extensions' => 'permessage-deflate');
    }

    my $pubsub = Mojo::Pg::PubSub->new(pg => $c->pg);

    $pubsub->listen(streams => sub($pubsub, $payload) {
        $c->send($payload);
    });

    $c->on(json => sub( $c, $message ) {
        if ( $message->{event} eq 'KEEPALIVE' ) {
        }
    });

    $c->on(finish => sub ($c, $code, $reason = undef) {
        $pubsub->unlisten('streams');
        $c->app->log->debug("WebSocket closed with status $code");
    });
}

1;
